@@ -17,7 +17,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 	"sigs.k8s.io/controller-runtime/pkg/manager/signals"
 
-	networkaddons "github.com/kubevirt/cluster-network-addons-operator/pkg/apis"
+	networkaddonsv1 "github.com/kubevirt/cluster-network-addons-operator/pkg/apis/networkaddonsoperator/v1"
 	"github.com/kubevirt/hyperconverged-cluster-operator/cmd/cmdcommon"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis"
 	hcoutil "github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
@@ -35,7 +35,7 @@ var (
 		apis.AddToScheme,
 		corev1.AddToScheme,
 		cdiv1beta1.AddToScheme,
-		networkaddons.AddToScheme,
+		networkaddonsv1.AddToScheme,
 		sspv1beta1.AddToScheme,
 		admissionregistrationv1.AddToScheme,
 		openshiftconfigv1.Install,
