@@ -3,42 +3,42 @@ package commonTestUtils
 import (
 	"context"
 	"fmt"
-	networkaddons "github.com/kubevirt/cluster-network-addons-operator/pkg/apis"
-	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis"
-	vmimportv1beta1 "github.com/kubevirt/vm-import-operator/pkg/apis/v2v/v1beta1"
-	consolev1 "github.com/openshift/api/console/v1"
-	routev1 "github.com/openshift/api/route/v1"
-	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	cdiv1beta1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
-	sspv1beta1 "kubevirt.io/ssp-operator/api/v1beta1"
-	"sigs.k8s.io/controller-runtime/pkg/log/zap"
-	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/common"
-	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
-
-	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/components"
-
-	hcov1beta1 "github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis/hco/v1beta1"
-	corev1 "k8s.io/api/core/v1"
-	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
-	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/client-go/kubernetes/scheme"
+	"github.com/go-logr/logr"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	gomegatypes "github.com/onsi/gomega/types"
+	consolev1 "github.com/openshift/api/console/v1"
+	imagev1 "github.com/openshift/api/image/v1"
+	routev1 "github.com/openshift/api/route/v1"
+	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
+	corev1 "k8s.io/api/core/v1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/client-go/kubernetes/scheme"
+	"sigs.k8s.io/controller-runtime/pkg/log/zap"
+	"sigs.k8s.io/controller-runtime/pkg/reconcile"
+
+	networkaddonsv1 "github.com/kubevirt/cluster-network-addons-operator/pkg/apis/networkaddonsoperator/v1"
+	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis"
+	hcov1beta1 "github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis/hco/v1beta1"
+	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/components"
+	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/common"
+	kubevirtcorev1 "kubevirt.io/api/core/v1"
+	cdiv1beta1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
+	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
+	sspv1beta1 "kubevirt.io/ssp-operator/api/v1beta1"
 )
 
 // Name and Namespace of our primary resource
 const (
-	Name            = "kubevirt-hyperconverged"
-	Namespace       = "kubevirt-hyperconverged"
-	ConversionImage = "quay.io/kubevirt/kubevirt-v2v-conversion:v2.0.0"
-	VmwareImage     = "quay.io/kubevirt/kubevirt-vmware:v2.0.0"
-	VirtioWinImage  = "quay.io/kubevirt/virtio-container-disk:v2.0.0"
+	Name           = "kubevirt-hyperconverged"
+	Namespace      = "kubevirt-hyperconverged"
+	VirtioWinImage = "quay.io/kubevirt/virtio-container-disk:v2.0.0"
 )
 
 var (
@@ -57,6 +57,14 @@ func NewHco() *hcov1beta1.HyperConverged {
 	return hco
 }
 
+func NewHcoNamespace() *corev1.Namespace {
+	return &corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: Namespace,
+		},
+	}
+}
+
 func NewReq(inst *hcov1beta1.HyperConverged) *common.HcoRequest {
 	return &common.HcoRequest{
 		Request:      TestRequest,
@@ -136,14 +144,15 @@ func GetScheme() *runtime.Scheme {
 
 	for _, f := range []func(*runtime.Scheme) error{
 		apis.AddToScheme,
+		kubevirtcorev1.AddToScheme,
 		cdiv1beta1.AddToScheme,
-		networkaddons.AddToScheme,
+		networkaddonsv1.AddToScheme,
 		sspv1beta1.AddToScheme,
-		vmimportv1beta1.AddToScheme,
 		consolev1.AddToScheme,
 		monitoringv1.AddToScheme,
 		apiextensionsv1.AddToScheme,
 		routev1.Install,
+		imagev1.Install,
 	} {
 		Expect(f(testScheme)).To(BeNil())
 	}
@@ -192,3 +201,78 @@ func (matcher *RepresentConditionMatcher) FailureMessage(actual interface{}) (me
 func (matcher *RepresentConditionMatcher) NegatedFailureMessage(actual interface{}) (message string) {
 	return fmt.Sprintf("Expected\n\t%#v\nnot to match the condition\n\t%#v", actual, matcher.expected)
 }
+
+// ClusterInfoMock mocks regular Openshift
+type ClusterInfoMock struct{}
+
+func (ClusterInfoMock) Init(_ context.Context, _ client.Client, _ logr.Logger) error {
+	return nil
+}
+func (ClusterInfoMock) IsOpenshift() bool {
+	return true
+}
+func (ClusterInfoMock) IsRunningLocally() bool {
+	return false
+}
+func (ClusterInfoMock) IsManagedByOLM() bool {
+	return true
+}
+func (ClusterInfoMock) IsControlPlaneHighlyAvailable() bool {
+	return true
+}
+func (ClusterInfoMock) IsInfrastructureHighlyAvailable() bool {
+	return true
+}
+func (ClusterInfoMock) GetDomain() string {
+	return "domain"
+}
+
+// ClusterInfoSNOMock mocks Openshift SNO
+type ClusterInfoSNOMock struct{}
+
+func (ClusterInfoSNOMock) Init(_ context.Context, _ client.Client, _ logr.Logger) error {
+	return nil
+}
+func (ClusterInfoSNOMock) IsOpenshift() bool {
+	return true
+}
+func (ClusterInfoSNOMock) IsRunningLocally() bool {
+	return false
+}
+func (ClusterInfoSNOMock) IsManagedByOLM() bool {
+	return true
+}
+func (ClusterInfoSNOMock) IsControlPlaneHighlyAvailable() bool {
+	return false
+}
+func (ClusterInfoSNOMock) IsInfrastructureHighlyAvailable() bool {
+	return false
+}
+func (ClusterInfoSNOMock) GetDomain() string {
+	return "domain"
+}
+
+// ClusterInfoSRCPHAIMock mocks Openshift with SingleReplica ControlPlane and HighAvailable Infrastructure
+type ClusterInfoSRCPHAIMock struct{}
+
+func (ClusterInfoSRCPHAIMock) Init(_ context.Context, _ client.Client, _ logr.Logger) error {
+	return nil
+}
+func (ClusterInfoSRCPHAIMock) IsOpenshift() bool {
+	return true
+}
+func (ClusterInfoSRCPHAIMock) IsRunningLocally() bool {
+	return false
+}
+func (ClusterInfoSRCPHAIMock) IsManagedByOLM() bool {
+	return true
+}
+func (ClusterInfoSRCPHAIMock) IsControlPlaneHighlyAvailable() bool {
+	return false
+}
+func (ClusterInfoSRCPHAIMock) IsInfrastructureHighlyAvailable() bool {
+	return true
+}
+func (ClusterInfoSRCPHAIMock) GetDomain() string {
+	return "domain"
+}
