@@ -4,28 +4,30 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	jsonpatch "github.com/evanphx/json-patch"
-	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/metrics"
-	consolev1 "github.com/openshift/api/console/v1"
-	routev1 "github.com/openshift/api/route/v1"
-	rbacv1 "k8s.io/api/rbac/v1"
-	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"os"
 	"reflect"
-	"strings"
+
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 
 	"github.com/blang/semver/v4"
-
+	jsonpatch "github.com/evanphx/json-patch"
 	"github.com/google/uuid"
+	consolev1 "github.com/openshift/api/console/v1"
+	imagev1 "github.com/openshift/api/image/v1"
+	routev1 "github.com/openshift/api/route/v1"
 	operatorhandler "github.com/operator-framework/operator-lib/handler"
+	"github.com/pkg/errors"
+	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
 	corev1 "k8s.io/api/core/v1"
+	rbacv1 "k8s.io/api/rbac/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	apimetav1 "k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/apimachinery/pkg/util/yaml"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -36,17 +38,15 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
 	networkaddonsv1 "github.com/kubevirt/cluster-network-addons-operator/pkg/apis/networkaddonsoperator/v1"
-	vmimportv1beta1 "github.com/kubevirt/vm-import-operator/pkg/apis/v2v/v1beta1"
-	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
-	kubevirtv1 "kubevirt.io/client-go/api/v1"
-	cdiv1beta1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
-	sspv1beta1 "kubevirt.io/ssp-operator/api/v1beta1"
-
 	hcov1beta1 "github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis/hco/v1beta1"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/common"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/operands"
+	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/metrics"
 	hcoutil "github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
-	version "github.com/kubevirt/hyperconverged-cluster-operator/version"
+	"github.com/kubevirt/hyperconverged-cluster-operator/version"
+	kubevirtcorev1 "kubevirt.io/api/core/v1"
+	cdiv1beta1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
+	sspv1beta1 "kubevirt.io/ssp-operator/api/v1beta1"
 )
 
 var (
@@ -75,6 +75,11 @@ const (
 
 	hcoVersionName    = "operator"
 	secondaryCRPrefix = "hco-controlled-cr-"
+
+	// These group are no longer supported. Use these constants to remove unused resources
+	v2vGroup = "v2v.kubevirt.io"
+
+	requestedStatusKey = "requested status"
 )
 
 // JSONPatchAnnotationNames - annotations used to patch operand CRs with unsupported/unofficial/hidden features.
@@ -86,12 +91,12 @@ var JSONPatchAnnotationNames = []string{
 }
 
 // RegisterReconciler creates a new HyperConverged Reconciler and registers it into manager.
-func RegisterReconciler(mgr manager.Manager, ci hcoutil.ClusterInfo) error {
-	return add(mgr, newReconciler(mgr, ci), ci)
+func RegisterReconciler(mgr manager.Manager, ci hcoutil.ClusterInfo, upgradeableCond hcoutil.Condition) error {
+	return add(mgr, newReconciler(mgr, ci, upgradeableCond), ci)
 }
 
 // newReconciler returns a new reconcile.Reconciler
-func newReconciler(mgr manager.Manager, ci hcoutil.ClusterInfo) reconcile.Reconciler {
+func newReconciler(mgr manager.Manager, ci hcoutil.ClusterInfo, upgradeableCond hcoutil.Condition) reconcile.Reconciler {
 
 	ownVersion := os.Getenv(hcoutil.HcoKvIoVersionName)
 	if ownVersion == "" {
@@ -99,16 +104,37 @@ func newReconciler(mgr manager.Manager, ci hcoutil.ClusterInfo) reconcile.Reconc
 	}
 
 	return &ReconcileHyperConverged{
-		client:         mgr.GetClient(),
-		scheme:         mgr.GetScheme(),
-		operandHandler: operands.NewOperandHandler(mgr.GetClient(), mgr.GetScheme(), ci.IsOpenshift(), hcoutil.GetEventEmitter()),
-		upgradeMode:    false,
-		ownVersion:     ownVersion,
-		eventEmitter:   hcoutil.GetEventEmitter(),
-		firstLoop:      true,
+		client:               mgr.GetClient(),
+		scheme:               mgr.GetScheme(),
+		operandHandler:       operands.NewOperandHandler(mgr.GetClient(), mgr.GetScheme(), ci.IsOpenshift(), hcoutil.GetEventEmitter()),
+		upgradeMode:          false,
+		ownVersion:           ownVersion,
+		eventEmitter:         hcoutil.GetEventEmitter(),
+		firstLoop:            true,
+		upgradeableCondition: upgradeableCond,
 	}
 }
 
+// newCRDremover returns a new CRDRemover
+func newCRDremover(client client.Client) *CRDRemover {
+	crdRemover := &CRDRemover{
+		client: client,
+		crdsToRemove: []schema.GroupKind{
+			// These are the v2v CRDs we have to remove moving to MTV
+			{Group: v2vGroup, Kind: "V2VVmware"},
+			{Group: v2vGroup, Kind: "OVirtProvider"},
+			{Group: v2vGroup, Kind: "VMImportConfig"},
+		},
+	}
+
+	// The list of related objects to remove is initialized empty;
+	// Once the corresponding CRD (and hence CR) is removed successfully,
+	// the CR can be removed from the list of related objects.
+	crdRemover.relatedObjectsToRemove = make([]schema.GroupKind, 0, len(crdRemover.crdsToRemove))
+
+	return crdRemover
+}
+
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
 func add(mgr manager.Manager, r reconcile.Reconciler, ci hcoutil.ClusterInfo) error {
 	// Create a new controller
@@ -136,12 +162,11 @@ func add(mgr manager.Manager, r reconcile.Reconciler, ci hcoutil.ClusterInfo) er
 	// When a new object got added here, it has also to be added to the custom cache
 	// managed by getNewManagerCache()
 	secondaryResources := []client.Object{
-		&kubevirtv1.KubeVirt{},
+		&kubevirtcorev1.KubeVirt{},
 		&cdiv1beta1.CDI{},
 		&networkaddonsv1.NetworkAddonsConfig{},
 		&sspv1beta1.SSP{},
 		&schedulingv1.PriorityClass{},
-		&vmimportv1beta1.VMImportConfig{},
 		&corev1.ConfigMap{},
 		&corev1.Service{},
 		&rbacv1.Role{},
@@ -154,6 +179,8 @@ func add(mgr manager.Manager, r reconcile.Reconciler, ci hcoutil.ClusterInfo) er
 			&monitoringv1.PrometheusRule{},
 			&routev1.Route{},
 			&consolev1.ConsoleCLIDownload{},
+			&imagev1.ImageStream{},
+			&corev1.Namespace{},
 		}...)
 	}
 
@@ -186,13 +213,14 @@ var _ reconcile.Reconciler = &ReconcileHyperConverged{}
 type ReconcileHyperConverged struct {
 	// This client, initialized using mgr.Client() above, is a split client
 	// that reads objects from the cache and writes to the apiserver
-	client         client.Client
-	scheme         *runtime.Scheme
-	operandHandler *operands.OperandHandler
-	upgradeMode    bool
-	ownVersion     string
-	eventEmitter   hcoutil.EventEmitter
-	firstLoop      bool
+	client               client.Client
+	scheme               *runtime.Scheme
+	operandHandler       *operands.OperandHandler
+	upgradeMode          bool
+	ownVersion           string
+	eventEmitter         hcoutil.EventEmitter
+	firstLoop            bool
+	upgradeableCondition hcoutil.Condition
 }
 
 // Reconcile reads that state of the cluster for a HyperConverged object and makes changes based on the state read
@@ -223,13 +251,26 @@ func (r *ReconcileHyperConverged) Reconcile(ctx context.Context, request reconci
 
 	// Fetch the HyperConverged instance
 	instance, err := r.getHyperConverged(hcoRequest)
+
+	hcoRequest.Instance = instance
+
 	if instance == nil {
+		// if the HyperConverged CR was deleted during an upgrade process, then this is not an upgrade anymore
+		r.upgradeMode = false
+		if err == nil {
+			err = r.setOperatorUpgradeableStatus(hcoRequest)
+		}
+
 		return reconcile.Result{}, err
 	}
-	hcoRequest.Instance = instance
 
 	if r.firstLoop {
 		r.firstLoopInitialization(hcoRequest)
+		if err := validateUpgradePatches(hcoRequest); err != nil {
+			logger.Error(err, "Failed validating upgrade patches file")
+			r.eventEmitter.EmitEvent(hcoRequest.Instance, corev1.EventTypeWarning, "Failed validating upgrade patches file", err.Error())
+			os.Exit(1)
+		}
 	}
 
 	result, err := r.doReconcile(hcoRequest)
@@ -238,6 +279,10 @@ func (r *ReconcileHyperConverged) Reconcile(ctx context.Context, request reconci
 		return result, err
 	}
 
+	if err = r.setOperatorUpgradeableStatus(hcoRequest); err != nil {
+		return reconcile.Result{}, err
+	}
+
 	requeue, err := r.updateHyperConverged(hcoRequest)
 	if requeue || apierrors.IsConflict(err) {
 		result.Requeue = true
@@ -306,13 +351,17 @@ func (r *ReconcileHyperConverged) doReconcile(req *common.HcoRequest) (reconcile
 		return r.ensureHcoDeleted(req)
 	}
 
+	applyDataImportSchedule(req)
+
 	// If the current version is not updated in CR ,then we're updating. This is also works when updating from
 	// an old version, since Status.Versions will be empty.
-	knownHcoVersion, _ := req.Instance.Status.GetVersion(hcoVersionName)
+	knownHcoVersion, _ := GetVersion(&req.Instance.Status, hcoVersionName)
 
+	// detect upgrade mode
 	if !r.upgradeMode && !init && knownHcoVersion != r.ownVersion {
-		r.upgradeMode = true
+		// get into upgrade mode
 
+		r.upgradeMode = true
 		r.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "UpgradeHCO", "Upgrading the HyperConverged to version "+r.ownVersion)
 		req.Logger.Info(fmt.Sprintf("Start upgrading from version %s to version %s", knownHcoVersion, r.ownVersion))
 	}
@@ -329,10 +378,34 @@ func (r *ReconcileHyperConverged) doReconcile(req *common.HcoRequest) (reconcile
 		crdStatusUpdated, err := r.updateCrdStoredVersions(req)
 		if err != nil {
 			return reconcile.Result{Requeue: true}, err
-		} else {
-			if crdStatusUpdated {
-				return reconcile.Result{Requeue: true}, nil
-			}
+		}
+
+		if crdStatusUpdated {
+			return reconcile.Result{Requeue: true}, nil
+		}
+
+		// Attempt to remove old CRDs and related objects:
+		// Directly removing v2v CRDs will be enough to trigger the
+		// removal of the corresponding CRs.
+		// vmimportconfigs CR is protected by "vm-import-finalizer"
+		// which is managed by vm-import-operator that should remove
+		// all of its operands before removing the finalizer so that the
+		// CR and then the CRD can be really deleted.
+		// We don't have any race condition here because the CRD will be
+		// deleted without blocking the caller.
+		// vm-import-operator pod on the other side is not deleted during the
+		// upgrade process, but only at the end of it as a consequence of
+		// the removal of the old CSV (vm-import-operator deployment has
+		// an owner reference on it) once the upgrade successfully completed.
+
+		cdrRemover := newCRDremover(r.client)
+		err = cdrRemover.Remove(req)
+		if err != nil {
+			return reconcile.Result{Requeue: init}, err
+		}
+		// if we still have something to remove, requeue to retry
+		if !cdrRemover.Done() {
+			return reconcile.Result{Requeue: true}, nil
 		}
 
 		modified, err := r.migrateBeforeUpgrade(req)
@@ -352,7 +425,6 @@ func (r *ReconcileHyperConverged) doReconcile(req *common.HcoRequest) (reconcile
 func (r *ReconcileHyperConverged) EnsureOperandAndComplete(req *common.HcoRequest, init bool) (reconcile.Result, error) {
 	if err := r.operandHandler.Ensure(req); err != nil {
 		r.updateConditions(req)
-		hcoutil.SetReady(false)
 		return reconcile.Result{Requeue: init}, nil
 	}
 
@@ -360,7 +432,6 @@ func (r *ReconcileHyperConverged) EnsureOperandAndComplete(req *common.HcoReques
 
 	// Requeue if we just created everything
 	if init {
-		hcoutil.SetReady(false)
 		return reconcile.Result{Requeue: true}, nil
 	}
 
@@ -480,7 +551,7 @@ func (r *ReconcileHyperConverged) validateNamespace(req *common.HcoRequest) (boo
 }
 
 func (r *ReconcileHyperConverged) setInitialConditions(req *common.HcoRequest) {
-	req.Instance.Status.UpdateVersion(hcoVersionName, r.ownVersion)
+	UpdateVersion(&req.Instance.Status, hcoVersionName, r.ownVersion)
 
 	req.Conditions.SetStatusCondition(metav1.Condition{
 		Type:               hcov1beta1.ConditionReconcileComplete,
@@ -536,6 +607,9 @@ func (r *ReconcileHyperConverged) ensureHcoDeleted(req *common.HcoRequest) (reco
 		req.Dirty = true
 		requeue = requeue || finDropped
 	}
+
+	// should never happen - we are dropping the wrong finalizer in checkFinalizers, that always called before this
+	// function
 	if hcoutil.ContainsString(req.Instance.ObjectMeta.Finalizers, badFinalizerName) {
 		req.Instance.ObjectMeta.Finalizers, finDropped = drop(req.Instance.ObjectMeta.Finalizers, badFinalizerName)
 		req.Dirty = true
@@ -761,7 +835,7 @@ func (r *ReconcileHyperConverged) completeReconciliation(req *common.HcoRequest)
 		// if in upgrade mode, and all the components are upgraded, and nothing pending to be written - upgrade is completed
 		if r.upgradeMode && req.ComponentUpgradeInProgress && !req.Dirty {
 			// update the new version only when upgrade is completed
-			req.Instance.Status.UpdateVersion(hcoVersionName, r.ownVersion)
+			UpdateVersion(&req.Instance.Status, hcoVersionName, r.ownVersion)
 			req.StatusDirty = true
 
 			r.upgradeMode = false
@@ -786,7 +860,6 @@ func (r *ReconcileHyperConverged) completeReconciliation(req *common.HcoRequest)
 		})
 	}
 
-	hcoutil.SetReady(hcoReady)
 	if hcoReady {
 		// If no operator whose conditions we are watching reports an error, then it is safe
 		// to set readiness.
@@ -852,11 +925,14 @@ func (r *ReconcileHyperConverged) detectTaintedConfiguration(req *common.HcoRequ
 		NumOfChanges := 0
 		jsonPatch, exists := req.Instance.ObjectMeta.Annotations[jpa]
 		if exists {
-			if NumOfChanges = getNumOfChangesJsonPatch(jsonPatch); NumOfChanges > 0 {
+			if NumOfChanges = getNumOfChangesJSONPatch(jsonPatch); NumOfChanges > 0 {
 				tainted = true
 			}
 		}
-		metrics.HcoMetrics.SetUnsafeModificationCount(NumOfChanges, jpa)
+		err := metrics.HcoMetrics.SetUnsafeModificationCount(NumOfChanges, jpa)
+		if err != nil {
+			req.Logger.Error(err, "couldn't update 'UnsafeModification' metric")
+		}
 	}
 
 	if tainted {
@@ -884,7 +960,7 @@ func (r *ReconcileHyperConverged) detectTaintedConfiguration(req *common.HcoRequ
 	}
 }
 
-func getNumOfChangesJsonPatch(jsonPatch string) int {
+func getNumOfChangesJSONPatch(jsonPatch string) int {
 	patches, err := jsonpatch.DecodePatch([]byte(jsonPatch))
 	if err != nil {
 		return 0
@@ -893,10 +969,6 @@ func getNumOfChangesJsonPatch(jsonPatch string) int {
 }
 
 func (r *ReconcileHyperConverged) firstLoopInitialization(request *common.HcoRequest) {
-	// Reload eventEmitter.
-	// The client should now find all the required resources.
-	r.eventEmitter.UpdateClient(request.Ctx, r.client, request.Logger)
-
 	// Initialize operand handler.
 	r.operandHandler.FirstUseInitiation(r.scheme, hcoutil.GetClusterInfo().IsOpenshift(), request.Instance)
 
@@ -904,23 +976,32 @@ func (r *ReconcileHyperConverged) firstLoopInitialization(request *common.HcoReq
 	r.firstLoop = false
 }
 
-// This function performs migrations before starting the upgrade process
-// return true if the HyperConverged CR was modified; else, return false
-//
-// If the kubevirt-config configMap exists:
-// 1. create a backup of the configMap, if not exists
-// 2. if the configMap includes the live migration configurations, and they are not match to the default values,
-//    update the HyperConverged CR
-// 3. remove the kubevirt-config configMap
-// 4. return true if the CR was modified in #2
-const (
-	kvCmName         = "kubevirt-config"
-	backupKvCmName   = kvCmName + "-backup"
-	imsCmName        = "v2v-vmware"
-	liveMigrationKey = "migrations"
-	vddkInitImakeKey = "vddk-init-image"
-	crdName          = "hyperconvergeds.hco.kubevirt.io"
-)
+func (r *ReconcileHyperConverged) setOperatorUpgradeableStatus(request *common.HcoRequest) error {
+	if hcoutil.GetClusterInfo().IsManagedByOLM() {
+
+		request.Logger.Info("setting the Upgradeable operator condition", requestedStatusKey, !r.upgradeMode)
+
+		msg := hcoutil.UpgradeableAllowMessage
+		status := metav1.ConditionTrue
+		reason := hcoutil.UpgradeableAllowReason
+
+		if r.upgradeMode {
+			msg = hcoutil.UpgradeableUpgradingMessage + r.ownVersion
+			status = metav1.ConditionFalse
+			reason = hcoutil.UpgradeableUpgradingReason
+		}
+
+		if err := r.upgradeableCondition.Set(request.Ctx, status, reason, msg); err != nil {
+			request.Logger.Error(err, "can't set the Upgradeable operator condition", requestedStatusKey, !r.upgradeMode)
+			return err
+		}
+
+	}
+
+	return nil
+}
+
+const crdName = "hyperconvergeds.hco.kubevirt.io"
 
 func (r *ReconcileHyperConverged) updateCrdStoredVersions(req *common.HcoRequest) (bool, error) {
 	versionsToBeRemoved := []string{"v1alpha1"}
@@ -934,7 +1015,7 @@ func (r *ReconcileHyperConverged) updateCrdStoredVersions(req *common.HcoRequest
 	}
 
 	needsUpdate := false
-	newStoredVersions := []string{}
+	var newStoredVersions []string
 	for _, vToBeRemoved := range versionsToBeRemoved {
 		for _, sVersion := range found.Status.StoredVersions {
 			if vToBeRemoved != sVersion {
@@ -959,118 +1040,25 @@ func (r *ReconcileHyperConverged) updateCrdStoredVersions(req *common.HcoRequest
 }
 
 func (r *ReconcileHyperConverged) migrateBeforeUpgrade(req *common.HcoRequest) (bool, error) {
-	kvConfigModified, err := r.migrateKvConfigurations(req)
-	if err != nil {
-		return false, err
-	}
-
-	cdiConfigModified, err := r.migrateCdiConfigurations(req)
-	if err != nil {
-		return false, err
-	}
-
-	imsConfigModified, err := r.migrateImsConfigurations(req)
-	if err != nil {
-		return false, err
-	}
-
-	defaultsAmended, err := r.amendBadDefaults(req)
-	if err != nil {
-		return false, err
-	}
-
 	upgradePatched, err := r.applyUpgradePatches(req)
 	if err != nil {
 		return false, err
 	}
 
-	return kvConfigModified || cdiConfigModified || imsConfigModified || defaultsAmended || upgradePatched, nil
-}
-
-func (r ReconcileHyperConverged) migrateKvConfigurations(req *common.HcoRequest) (bool, error) {
-	cm, err := r.getCm(kvCmName, req)
+	err = r.removeOldMetricsObjs(req)
 	if err != nil {
 		return false, err
-	} else if cm == nil {
-		return false, nil
 	}
 
-	if err = r.makeCmBackup(cm, backupKvCmName, req); err != nil {
-		return false, err
-	}
+	removeOldQuickStartGuides(req, r.client, r.operandHandler.GetQuickStartNames())
 
-	modified := adoptOldKvConfigs(req, cm)
-
-	if !modified {
-		err = r.removeConfigMap(req, cm, kvCmName)
-		if err != nil {
-			return false, err
-		}
-	}
-
-	return modified, nil
-}
-
-func (r ReconcileHyperConverged) migrateCdiConfigurations(req *common.HcoRequest) (bool, error) {
-	cdi := operands.NewCDIWithNameOnly(req.Instance)
-
-	err := hcoutil.GetRuntimeObject(req.Ctx, r.client, cdi, req.Logger)
-	if err != nil {
-		if apierrors.IsNotFound(err) {
-			return false, nil
-		}
-		return false, err
-	}
-
-	return adoptCdiConfigs(req, cdi.Spec.Config), nil
-}
-
-func (r ReconcileHyperConverged) migrateImsConfigurations(req *common.HcoRequest) (bool, error) {
-	req.Logger.Info("read IMS configmap")
-	cm := &corev1.ConfigMap{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      imsCmName,
-			Namespace: req.Namespace,
-		},
-	}
-
-	if err := hcoutil.GetRuntimeObject(req.Ctx, r.client, cm, req.Logger); err != nil {
-		if apierrors.IsNotFound(err) {
-			req.Logger.Info("IMS configmap already removed")
-			return false, nil
-		}
-		req.Logger.Info("failed to get IMS configmap", "error", err.Error())
-		return false, err
-	}
-
-	modified := false
-	vddkInitImage, ok := cm.Data[vddkInitImakeKey]
-	if ok {
-		if req.Instance.Spec.VddkInitImage == nil {
-			req.Logger.Info("updating the HyperConverged CR from the IMS configMap")
-			req.Instance.Spec.VddkInitImage = &vddkInitImage
-			req.Dirty = true
-			modified = true
-		}
-	}
-	return modified, nil
-}
-
-func (r ReconcileHyperConverged) amendBadDefaults(req *common.HcoRequest) (bool, error) {
-	modified := false
-	if req.Instance.Spec.LiveMigrationConfig.BandwidthPerMigration != nil && *req.Instance.Spec.LiveMigrationConfig.BandwidthPerMigration == "64Mi" {
-		req.Logger.Info("rejecting spec.livemigrationconfig.bandwidthpermigration==64Mi because it was a bad default value")
-		req.Instance.Spec.LiveMigrationConfig.BandwidthPerMigration = nil
-		modified = true
-		req.Dirty = true
-	}
-	return modified, nil
+	return upgradePatched, nil
 }
 
 func (r ReconcileHyperConverged) applyUpgradePatches(req *common.HcoRequest) (bool, error) {
 	modified := false
 
-	knownHcoVersion, _ := req.Instance.Status.GetVersion(hcoVersionName)
+	knownHcoVersion, _ := GetVersion(&req.Instance.Status, hcoVersionName)
 	if knownHcoVersion == "" {
 		knownHcoVersion = "0.0.0"
 	}
@@ -1092,6 +1080,13 @@ func (r ReconcileHyperConverged) applyUpgradePatches(req *common.HcoRequest) (bo
 		}
 	}
 
+	for _, p := range hcoUpgradeChanges.ObjectsToBeRemoved {
+		removed, err := r.removeLeftover(req, knownHcoSV, p)
+		if err != nil {
+			return removed, err
+		}
+	}
+
 	tmpInstance := &hcov1beta1.HyperConverged{}
 	err = json.Unmarshal(hcoJson, tmpInstance)
 	if err != nil {
@@ -1116,6 +1111,10 @@ func (r ReconcileHyperConverged) applyUpgradePatch(req *common.HcoRequest, hcoJs
 		req.Logger.Info("applying upgrade patch", "knownHcoSV", knownHcoSV, "affectedRange", p.SemverRange, "patches", p.JSONPatch)
 		patchedBytes, err := p.JSONPatch.Apply(hcoJson)
 		if err != nil {
+			if errors.Cause(err) == jsonpatch.ErrTestFailed {
+				return hcoJson, nil
+			}
+
 			return hcoJson, err
 		}
 		return patchedBytes, nil
@@ -1123,148 +1122,173 @@ func (r ReconcileHyperConverged) applyUpgradePatch(req *common.HcoRequest, hcoJs
 	return hcoJson, nil
 }
 
-func (r *ReconcileHyperConverged) removeConfigMap(req *common.HcoRequest, cm *corev1.ConfigMap, cmName string) error {
-	req.Logger.Info("removing the kubevirt configMap")
-	err := hcoutil.ComponentResourceRemoval(req.Ctx, r.client, cm, req.Name, req.Logger, false, true)
+func (r ReconcileHyperConverged) removeLeftover(req *common.HcoRequest, knownHcoSV semver.Version, p objectToBeRemoved) (bool, error) {
+
+	affectedRange, err := semver.ParseRange(p.SemverRange)
 	if err != nil {
-		return err
+		return false, err
+	}
+	if affectedRange(knownHcoSV) {
+		removeRelatedObject(req, p.GroupVersionKind, p.ObjectKey)
+		u := &unstructured.Unstructured{}
+		u.SetGroupVersionKind(p.GroupVersionKind)
+		gerr := r.client.Get(req.Ctx, p.ObjectKey, u)
+		if gerr != nil {
+			if apierrors.IsNotFound(gerr) {
+				return false, nil
+			} else {
+				req.Logger.Error(gerr, "failed looking for leftovers", "objectToBeRemoved", p)
+				return false, gerr
+			}
+		}
+		return r.deleteObj(req, u, false)
+
+	}
+	return false, nil
+}
+
+var (
+	operatorMetrics = "hyperconverged-cluster-operator-metrics"
+	webhookMetrics  = "hyperconverged-cluster-webhook-metrics"
+
+	oldMetricsObjects map[string]client.Object
+)
+
+func initOldMetricsObjects(req *common.HcoRequest) {
+	if oldMetricsObjects == nil {
+		oldMetricsObjects = map[string]client.Object{
+			"operatorService": &corev1.Service{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      operatorMetrics,
+					Namespace: req.Namespace,
+				},
+			},
+			"webhookService": &corev1.Service{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      webhookMetrics,
+					Namespace: req.Namespace,
+				},
+			},
+			"operatorEndpoint": &corev1.Endpoints{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      operatorMetrics,
+					Namespace: req.Namespace,
+				},
+			},
+		}
+	}
+}
+
+func (r ReconcileHyperConverged) removeOldMetricsObjs(req *common.HcoRequest) error {
+	initOldMetricsObjects(req)
+
+	for name, object := range oldMetricsObjects {
+		removed, err := r.deleteObj(req, object, true)
+
+		if err != nil {
+			return err
+		}
+
+		if removed {
+			delete(oldMetricsObjects, name)
+		}
 	}
 
-	r.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "Killing", fmt.Sprintf("Removed ConfigMap %s", cmName))
+	return nil
+}
 
+func (r *ReconcileHyperConverged) deleteObj(req *common.HcoRequest, obj client.Object, protectNonHCOObjects bool) (bool, error) {
+	removed, err := hcoutil.EnsureDeleted(req.Ctx, r.client, obj, req.Instance.Name, req.Logger, false, false, protectNonHCOObjects)
+
+	if err != nil {
+		req.Logger.Error(
+			err,
+			fmt.Sprintf("failed to delete %s", obj.GetObjectKind().GroupVersionKind().Kind),
+			"name",
+			obj.GetName(),
+		)
+
+		return removed, err
+	}
+
+	r.eventEmitter.EmitEvent(
+		req.Instance, corev1.EventTypeNormal, "Killing",
+		fmt.Sprintf("Removed %s %s", obj.GetName(), obj.GetObjectKind().GroupVersionKind().Kind),
+	)
+
+	return removed, nil
+}
+
+func removeOldQuickStartGuides(req *common.HcoRequest, cl client.Client, requiredQSList []string) {
+	existingQSList := &consolev1.ConsoleQuickStartList{}
+	req.Logger.Info("reading quickstart guides")
+	err := cl.List(req.Ctx, existingQSList, client.MatchingLabels{hcoutil.AppLabelManagedBy: hcoutil.OperatorName})
+	if err != nil {
+		req.Logger.Error(err, "failed to read list of quickstart guides")
+		return
+	}
+
+	var existingQSNames map[string]consolev1.ConsoleQuickStart
+	if len(existingQSList.Items) > 0 {
+		existingQSNames = make(map[string]consolev1.ConsoleQuickStart)
+		for _, qs := range existingQSList.Items {
+			existingQSNames[qs.Name] = qs
+		}
+
+		for name, existQs := range existingQSNames {
+			if !hcoutil.ContainsString(requiredQSList, name) {
+				req.Logger.Info("deleting ConsoleQuickStart", "name", name)
+				if _, err = hcoutil.EnsureDeleted(req.Ctx, cl, &existQs, req.Instance.Name, req.Logger, false, false, true); err != nil {
+					req.Logger.Error(err, "failed to delete ConsoleQuickStart", "name", name)
+				}
+			}
+		}
+
+		removeRelatedQSObjects(req, requiredQSList)
+	}
+}
+
+// removeRelatedQSObjects removes old quickstart from the related object list
+// can't use the removeRelatedObject function because the status not get updated during each reconcile loop,
+// but the old qs already removed (above) so you loos track of it. That why we must re-check all the qs names
+func removeRelatedQSObjects(req *common.HcoRequest, requiredNames []string) {
 	refs := make([]corev1.ObjectReference, 0, len(req.Instance.Status.RelatedObjects))
+	foundOldQs := false
+
 	for _, obj := range req.Instance.Status.RelatedObjects {
-		if obj.Kind == "ConfigMap" && obj.Name == cmName {
+		if obj.Kind == "ConsoleQuickStart" && !hcoutil.ContainsString(requiredNames, obj.Name) {
+			foundOldQs = true
 			continue
 		}
 		refs = append(refs, obj)
 	}
 
-	req.Instance.Status.RelatedObjects = refs
-	req.StatusDirty = true
-
-	return nil
-}
-
-func (r *ReconcileHyperConverged) getCm(cmName string, req *common.HcoRequest) (*corev1.ConfigMap, error) {
-	cm := &corev1.ConfigMap{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      cmName,
-			Namespace: req.Namespace,
-		},
+	if foundOldQs {
+		req.Instance.Status.RelatedObjects = refs
+		req.StatusDirty = true
 	}
 
-	if err := hcoutil.GetRuntimeObject(req.Ctx, r.client, cm, req.Logger); err != nil {
-		if apierrors.IsNotFound(err) {
-			req.Logger.Info(fmt.Sprintf("%s configmap already removed", cmName))
-			return nil, nil
+}
+
+func removeRelatedObject(req *common.HcoRequest, gvk schema.GroupVersionKind, objectKey types.NamespacedName) {
+	refs := make([]corev1.ObjectReference, 0, len(req.Instance.Status.RelatedObjects))
+	foundRO := false
+
+	for _, obj := range req.Instance.Status.RelatedObjects {
+		apiVersion, kind := gvk.ToAPIVersionAndKind()
+		if obj.APIVersion == apiVersion && obj.Kind == kind && obj.Namespace == objectKey.Namespace && obj.Name == objectKey.Name {
+			foundRO = true
+			continue
 		}
-		req.Logger.Info(fmt.Sprintf("failed to get %s configmap", cmName), "error", err.Error())
-		return nil, err
+		refs = append(refs, obj)
 	}
 
-	return cm, nil
-}
-
-func (r *ReconcileHyperConverged) makeCmBackup(cm *corev1.ConfigMap, backupName string, req *common.HcoRequest) error {
-	backupCm := &corev1.ConfigMap{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      backupName,
-			Namespace: cm.Namespace,
-			Labels:    cm.Labels,
-		},
-		Data: cm.Data,
+	if foundRO {
+		req.Instance.Status.RelatedObjects = refs
+		req.StatusDirty = true
+		req.Logger.Info("Removed relatedObject entry for", "gvk", gvk, "objectKey", objectKey)
 	}
 
-	req.Logger.Info(fmt.Sprintf("creating %s configmap backup", backupName))
-	if err := r.client.Create(req.Ctx, backupCm); err != nil {
-		if apierrors.IsAlreadyExists(err) {
-			req.Logger.Info(fmt.Sprintf("%s configmap backup already exists", backupName))
-		} else {
-			req.Logger.Info(fmt.Sprintf("failed to create %s configmap backup", backupName), "error", err.Error())
-			return err
-		}
-	} else {
-		r.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "Created", fmt.Sprintf("Created ConfigMap %s", backupName))
-	}
-
-	return nil
-}
-
-// Read the old KubeVit configuration from the config map, and move them to the HyperConverged CR
-//
-// In case of wrong foramt of the configmap, the HCO ignores this error (but print it to the log) in order to prevent
-// an infinite loop (returning error will cause the same error again and again, and the only way to stop the loop
-// is to manually fix or delete the wrong configMap).
-func adoptOldKvConfigs(req *common.HcoRequest, cm *corev1.ConfigMap) bool {
-	modified := false
-	kvLiveMigrationConfig, ok := cm.Data[liveMigrationKey]
-	if !ok {
-		return false
-	}
-	kvCmLiveMigrationConfig := hcov1beta1.LiveMigrationConfigurations{}
-	req.Instance.Spec.LiveMigrationConfig.DeepCopyInto(&kvCmLiveMigrationConfig)
-	err := yaml.NewYAMLOrJSONDecoder(strings.NewReader(kvLiveMigrationConfig), 1024).Decode(&kvCmLiveMigrationConfig)
-	if err != nil {
-		req.Logger.Error(err, "Failed to read the KubeVirt ConfigMap, and its content was ignored. This ConfigMap will be deleted. The backup ConfigMap called "+backupKvCmName)
-		return false
-	}
-
-	if !reflect.DeepEqual(req.Instance.Spec.LiveMigrationConfig, kvCmLiveMigrationConfig) {
-		req.Logger.Info("updating the HyperConverged CR from the KubeVirt configMap")
-		kvConfigMapToHyperConvergedCr(req, kvCmLiveMigrationConfig)
-
-		modified = true
-		req.Dirty = true
-	}
-
-	return modified
-}
-
-func adoptCdiConfigs(req *common.HcoRequest, cdiCfg *cdiv1beta1.CDIConfigSpec) bool {
-	modified := false
-	if cdiCfg != nil {
-		if req.Instance.Spec.ScratchSpaceStorageClass == nil && cdiCfg.ScratchSpaceStorageClass != nil {
-			req.Instance.Spec.ScratchSpaceStorageClass = new(string)
-			*req.Instance.Spec.ScratchSpaceStorageClass = *cdiCfg.ScratchSpaceStorageClass
-			modified = true
-		}
-
-		if cdiCfg.PodResourceRequirements != nil {
-			if req.Instance.Spec.ResourceRequirements == nil {
-				req.Instance.Spec.ResourceRequirements = &hcov1beta1.OperandResourceRequirements{}
-			}
-
-			if req.Instance.Spec.ResourceRequirements.StorageWorkloads == nil {
-				req.Instance.Spec.ResourceRequirements.StorageWorkloads = cdiCfg.PodResourceRequirements.DeepCopy()
-				modified = true
-			}
-		}
-	}
-
-	if modified {
-		req.Dirty = true
-	}
-
-	return modified
-}
-
-func kvConfigMapToHyperConvergedCr(req *common.HcoRequest, kvCmLMConfig hcov1beta1.LiveMigrationConfigurations) {
-	if kvCmLMConfig.BandwidthPerMigration != nil {
-		req.Instance.Spec.LiveMigrationConfig.BandwidthPerMigration = kvCmLMConfig.BandwidthPerMigration
-	}
-	if kvCmLMConfig.CompletionTimeoutPerGiB != nil {
-		req.Instance.Spec.LiveMigrationConfig.CompletionTimeoutPerGiB = kvCmLMConfig.CompletionTimeoutPerGiB
-	}
-	if kvCmLMConfig.ParallelMigrationsPerCluster != nil {
-		req.Instance.Spec.LiveMigrationConfig.ParallelMigrationsPerCluster = kvCmLMConfig.ParallelMigrationsPerCluster
-	}
-	if kvCmLMConfig.ParallelOutboundMigrationsPerNode != nil {
-		req.Instance.Spec.LiveMigrationConfig.ParallelOutboundMigrationsPerNode = kvCmLMConfig.ParallelOutboundMigrationsPerNode
-	}
-	if kvCmLMConfig.ProgressTimeout != nil {
-		req.Instance.Spec.LiveMigrationConfig.ProgressTimeout = kvCmLMConfig.ProgressTimeout
-	}
 }
 
 // getHyperConvergedNamespacedName returns the name/namespace of the HyperConverged resource
@@ -1300,7 +1324,7 @@ func getSecondaryCRPlaceholder() (types.NamespacedName, error) {
 }
 
 func drop(slice []string, s string) ([]string, bool) {
-	newSlice := []string{}
+	var newSlice []string
 	dropped := false
 	for _, element := range slice {
 		if element != s {
