@@ -4,22 +4,19 @@ import (
 	"errors"
 	"reflect"
 
-	conditionsv1 "github.com/openshift/custom-resource-status/conditions/v1"
-	objectreferencesv1 "github.com/openshift/custom-resource-status/objectreferences/v1"
+	log "github.com/go-logr/logr"
+
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
-	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/client-go/tools/reference"
-	cdiv1beta1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"sigs.k8s.io/controller-runtime/pkg/client"
-	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 
 	hcov1beta1 "github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis/hco/v1beta1"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/common"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
 	hcoutil "github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
+	cdiv1beta1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 )
 
 const (
@@ -60,8 +57,8 @@ func (h *cdiHooks) getFullCr(hc *hcov1beta1.HyperConverged) (client.Object, erro
 	return h.cache, nil
 }
 func (h cdiHooks) getEmptyCr() client.Object { return &cdiv1beta1.CDI{} }
-func (h cdiHooks) getConditions(cr runtime.Object) []conditionsv1.Condition {
-	return cr.(*cdiv1beta1.CDI).Status.Conditions
+func (h cdiHooks) getConditions(cr runtime.Object) []metav1.Condition {
+	return osConditionsToK8s(cr.(*cdiv1beta1.CDI).Status.Conditions)
 }
 func (h cdiHooks) checkComponentVersion(cr runtime.Object) bool {
 	found := cr.(*cdiv1beta1.CDI)
@@ -105,22 +102,11 @@ func getDefaultFeatureGates() []string {
 	return []string{HonorWaitForFirstConsumerGate}
 }
 
-func (h *cdiHooks) postFound(req *common.HcoRequest, exists runtime.Object) error {
-	err := h.ensureKubeVirtStorageRole(req)
-	if err != nil {
-		return err
-	}
-
-	err = h.ensureKubeVirtStorageRoleBinding(req)
-	if err != nil {
-		return err
-	}
-
-	return nil
-}
-
 func NewCDI(hc *hcov1beta1.HyperConverged, opts ...string) (*cdiv1beta1.CDI, error) {
 	uninstallStrategy := cdiv1beta1.CDIUninstallStrategyBlockUninstallIfWorkloadsExist
+	if hc.Spec.UninstallStrategy != nil && *hc.Spec.UninstallStrategy == hcov1beta1.HyperConvergedUninstallStrategyRemoveWorkloads {
+		uninstallStrategy = cdiv1beta1.CDIUninstallStrategyRemoveWorkloads
+	}
 
 	spec := cdiv1beta1.CDISpec{
 		UninstallStrategy: &uninstallStrategy,
@@ -187,120 +173,6 @@ func NewCDIWithNameOnly(hc *hcov1beta1.HyperConverged, opts ...string) *cdiv1bet
 	}
 }
 
-func (h *cdiHooks) ensureKubeVirtStorageRole(req *common.HcoRequest) error {
-	kubevirtStorageRole := NewKubeVirtStorageRoleForCR(req.Instance, req.Namespace, h.Scheme)
-
-	found := &rbacv1.Role{}
-	err := h.Client.Get(req.Ctx, client.ObjectKeyFromObject(kubevirtStorageRole), found)
-	if apierrors.IsNotFound(err) {
-		req.Logger.Info("Creating kubevirt storage role")
-		return h.Client.Create(req.Ctx, kubevirtStorageRole)
-	}
-
-	if err != nil {
-		return err
-	}
-
-	if !reflect.DeepEqual(found.Labels, kubevirtStorageRole.Labels) {
-		req.Logger.Info("Updating KubeVirt storage role for labels")
-		util.DeepCopyLabels(&kubevirtStorageRole.ObjectMeta, &found.ObjectMeta)
-		return h.Client.Update(req.Ctx, found)
-	}
-
-	req.Logger.Info("KubeVirt storage role already exists", "KubeVirtConfig.Namespace", found.Namespace, "KubeVirtConfig.Name", found.Name)
-	// Add it to the list of RelatedObjects if found
-	objectRef, err := reference.GetReference(h.Scheme, found)
-	if err != nil {
-		return err
-	}
-	if err = objectreferencesv1.SetObjectReference(&req.Instance.Status.RelatedObjects, *objectRef); err != nil {
-		return err
-	}
-
-	return nil
-}
-
-func (h *cdiHooks) ensureKubeVirtStorageRoleBinding(req *common.HcoRequest) error {
-	kubevirtStorageRoleBinding := NewKubeVirtStorageRoleBindingForCR(req.Instance, req.Namespace, h.Scheme)
-
-	found := &rbacv1.RoleBinding{}
-	err := h.Client.Get(req.Ctx, client.ObjectKeyFromObject(kubevirtStorageRoleBinding), found)
-	if err != nil && apierrors.IsNotFound(err) {
-		req.Logger.Info("Creating kubevirt storage rolebinding")
-		return h.Client.Create(req.Ctx, kubevirtStorageRoleBinding)
-	}
-
-	if err != nil {
-		return err
-	}
-
-	if !reflect.DeepEqual(found.Labels, kubevirtStorageRoleBinding.Labels) {
-		req.Logger.Info("Updating KubeVirt storage rolebinding for labels")
-		util.DeepCopyLabels(&kubevirtStorageRoleBinding.ObjectMeta, &found.ObjectMeta)
-		return h.Client.Update(req.Ctx, found)
-	}
-
-	req.Logger.Info("KubeVirt storage rolebinding already exists", "KubeVirtConfig.Namespace", found.Namespace, "KubeVirtConfig.Name", found.Name)
-	// Add it to the list of RelatedObjects if found
-	objectRef, err := reference.GetReference(h.Scheme, found)
-	if err != nil {
-		return err
-	}
-	err = objectreferencesv1.SetObjectReference(&req.Instance.Status.RelatedObjects, *objectRef)
-	if err != nil {
-		return err
-	}
-
-	return nil
-}
-
-func NewKubeVirtStorageRoleForCR(cr *hcov1beta1.HyperConverged, namespace string, scheme *runtime.Scheme) *rbacv1.Role {
-	role := &rbacv1.Role{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      cdiRoleName,
-			Labels:    getLabels(cr, hcoutil.AppComponentStorage),
-			Namespace: namespace,
-		},
-		Rules: []rbacv1.PolicyRule{
-			{
-				APIGroups:     []string{""},
-				Resources:     []string{"configmaps"},
-				ResourceNames: []string{"kubevirt-storage-class-defaults", "v2v-vmware"},
-				Verbs:         []string{"get", "watch", "list"},
-			},
-		},
-	}
-
-	_ = controllerutil.SetControllerReference(cr, role, scheme)
-	return role
-}
-
-func NewKubeVirtStorageRoleBindingForCR(cr *hcov1beta1.HyperConverged, namespace string, scheme *runtime.Scheme) *rbacv1.RoleBinding {
-	roleBinding := &rbacv1.RoleBinding{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      cdiRoleName,
-			Labels:    getLabels(cr, hcoutil.AppComponentStorage),
-			Namespace: namespace,
-		},
-		RoleRef: rbacv1.RoleRef{
-			APIGroup: "rbac.authorization.k8s.io",
-			Kind:     "Role",
-			Name:     cdiRoleName,
-		},
-		Subjects: []rbacv1.Subject{
-			{
-				APIGroup: "rbac.authorization.k8s.io",
-				Kind:     "Group",
-				Name:     "system:authenticated",
-			},
-		},
-	}
-
-	_ = controllerutil.SetControllerReference(cr, roleBinding, scheme)
-
-	return roleBinding
-}
-
 // ************** CDI Storage Config Handler **************
 type storageConfigHandler genericOperand
 
@@ -320,8 +192,7 @@ type storageConfigHooks struct{}
 func (h storageConfigHooks) getFullCr(hc *hcov1beta1.HyperConverged) (client.Object, error) {
 	return NewKubeVirtStorageConfigForCR(hc, hc.Namespace), nil
 }
-func (h storageConfigHooks) getEmptyCr() client.Object                              { return &corev1.ConfigMap{} }
-func (h storageConfigHooks) postFound(_ *common.HcoRequest, _ runtime.Object) error { return nil }
+func (h storageConfigHooks) getEmptyCr() client.Object { return &corev1.ConfigMap{} }
 func (h storageConfigHooks) getObjectMeta(cr runtime.Object) *metav1.ObjectMeta {
 	return &cr.(*corev1.ConfigMap).ObjectMeta
 }
@@ -383,3 +254,58 @@ func NewKubeVirtStorageConfigForCR(cr *hcov1beta1.HyperConverged, namespace stri
 		},
 	}
 }
+
+// ************** Config Reader Role Handler **************
+func NewConfigReaderRoleHandler(_ log.Logger, Client client.Client, Scheme *runtime.Scheme, hc *hcov1beta1.HyperConverged) ([]Operand, error) {
+	cdiConfigReaderRole := NewCdiConfigReaderRole(hc)
+
+	return []Operand{newRoleHandler(Client, Scheme, cdiConfigReaderRole)}, nil
+
+}
+
+// ************** Config Reader Role Binding Handler **************
+func newConfigReaderRoleBindingHandler(_ log.Logger, Client client.Client, Scheme *runtime.Scheme, hc *hcov1beta1.HyperConverged) ([]Operand, error) {
+	cdiConfigReaderRoleBinding := NewCdiConfigReaderRoleBinding(hc)
+
+	return []Operand{newRoleBindingHandler(Client, Scheme, cdiConfigReaderRoleBinding)}, nil
+}
+
+func NewCdiConfigReaderRole(hc *hcov1beta1.HyperConverged) *rbacv1.Role {
+	return &rbacv1.Role{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cdiRoleName,
+			Labels:    getLabels(hc, hcoutil.AppComponentStorage),
+			Namespace: hc.Namespace,
+		},
+		Rules: []rbacv1.PolicyRule{
+			{
+				APIGroups:     []string{""},
+				Resources:     []string{"configmaps"},
+				ResourceNames: []string{"kubevirt-storage-class-defaults"},
+				Verbs:         []string{"get", "watch", "list"},
+			},
+		},
+	}
+}
+
+func NewCdiConfigReaderRoleBinding(hc *hcov1beta1.HyperConverged) *rbacv1.RoleBinding {
+	return &rbacv1.RoleBinding{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cdiRoleName,
+			Labels:    getLabels(hc, hcoutil.AppComponentStorage),
+			Namespace: hc.Namespace,
+		},
+		RoleRef: rbacv1.RoleRef{
+			APIGroup: "rbac.authorization.k8s.io",
+			Kind:     "Role",
+			Name:     cdiRoleName,
+		},
+		Subjects: []rbacv1.Subject{
+			{
+				APIGroup: "rbac.authorization.k8s.io",
+				Kind:     "Group",
+				Name:     "system:authenticated",
+			},
+		},
+	}
+}
