@@ -24,14 +24,7 @@ import (
 	"errors"
 	"flag"
 	"fmt"
-	"github.com/blang/semver/v4"
-	"github.com/ghodss/yaml"
-	"github.com/imdario/mergo"
-	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/components"
-	hcoutil "github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
-	"github.com/kubevirt/hyperconverged-cluster-operator/tools/util"
 	"io/ioutil"
-	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions"
 	"log"
 	"os"
 	"path/filepath"
@@ -39,16 +32,32 @@ import (
 	"sort"
 	"strings"
 
+	"github.com/blang/semver/v4"
+	"github.com/ghodss/yaml"
+	"github.com/imdario/mergo"
+	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions"
+
+	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/components"
+	hcoutil "github.com/kubevirt/hyperconverged-cluster-operator/pkg/util"
+	"github.com/kubevirt/hyperconverged-cluster-operator/tools/util"
+
 	csvv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
 	corev1 "k8s.io/api/core/v1"
 )
 
 const (
-	operatorName          = "kubevirt-hyperconverged-operator"
-	CSVMode               = "CSV"
-	CRDMode               = "CRDs"
-	almExamplesAnnotation = "alm-examples"
-	validOutputModes      = CSVMode + "|" + CRDMode
+	operatorName            = "kubevirt-hyperconverged-operator"
+	CSVMode                 = "CSV"
+	CRDMode                 = "CRDs"
+	almExamplesAnnotation   = "alm-examples"
+	validOutputModes        = CSVMode + "|" + CRDMode
+	supported               = "supported"
+	operatorFrameworkPrefix = "operatorframework.io/"
+)
+
+var (
+	supported_archs = []string{"arch.amd64"}
+	supported_os    = []string{"os.linux"}
 )
 
 type EnvVarFlags []corev1.EnvVar
@@ -79,12 +88,9 @@ var (
 	cdiCsv              = flag.String("cdi-csv", "", "Containerized Data Importer CSV String")
 	nmoCsv              = flag.String("nmo-csv", "", "Node Maintenance Operator CSV String")
 	hppCsv              = flag.String("hpp-csv", "", "HostPath Provisioner Operator CSV String")
-	vmImportCsv         = flag.String("vmimport-csv", "", "Virtual Machine Import Operator CSV String")
 	operatorImage       = flag.String("operator-image-name", "", "HyperConverged Cluster Operator image")
 	webhookImage        = flag.String("webhook-image-name", "", "HyperConverged Cluster Webhook image")
 	cliDownloadsImage   = flag.String("cli-downloads-image-name", "", "Downloads Server image")
-	imsConversionImage  = flag.String("ims-conversion-image-name", "", "IMS conversion image")
-	imsVMWareImage      = flag.String("ims-vmware-image-name", "", "IMS VMWare image")
 	kvVirtIOWinImage    = flag.String("kv-virtiowin-image-name", "", "KubeVirt VirtIO Win image")
 	smbios              = flag.String("smbios", "", "Custom SMBIOS string for KubeVirt ConfigMap")
 	machinetype         = flag.String("machinetype", "", "Custom MACHINETYPE string for KubeVirt ConfigMap")
@@ -109,7 +115,6 @@ var (
 	sspVersion                    = flag.String("ssp-version", "", "SSP operator version")
 	nmoVersion                    = flag.String("nmo-version", "", "NM operator version")
 	hppoVersion                   = flag.String("hppo-version", "", "HPP operator version")
-	vmImportVersion               = flag.String("vm-import-version", "", "VM-Import operator version")
 	apiSources                    = flag.String("api-sources", cwd+"/...", "Project sources")
 	enableUniqueSemver            = flag.Bool("enable-unique-version", false, "Insert a skipRange annotation to support unique semver in the CSV")
 	envVars                       EnvVarFlags
@@ -121,14 +126,6 @@ func genHcoCrds() error {
 		return err
 	}
 
-	if err := util.MarshallObject(components.GetV2VCRD(), os.Stdout); err != nil {
-		return err
-	}
-
-	if err := util.MarshallObject(components.GetV2VOvirtProviderCRD(), os.Stdout); err != nil {
-		return err
-	}
-
 	return nil
 }
 
@@ -316,6 +313,8 @@ func getHcoCsv() {
 		csvBase.Spec.DisplayName = *specDisplayName
 	}
 
+	setSupported(csvBase)
+
 	applyOverrides(csvBase)
 
 	csvBase.Spec.RelatedImages = sortRelatedImages(csvBase.Spec.RelatedImages)
@@ -348,16 +347,19 @@ func processCsvs(componentsWithCsvs []util.CsvWithComponent, installStrategyBase
 	}
 }
 
+var csvNames = []string{"CNA", "KubeVirt", "SSP", "CDI", "NMO", "HPP", "VM Import"}
+
 func processOneCsv(c util.CsvWithComponent, i int, installStrategyBase *csvv1alpha1.StrategyDetailsDeployment, csvBase *csvv1alpha1.ClusterServiceVersion, ris *[]csvv1alpha1.RelatedImage) {
+	csvName := csvNames[i]
+
 	if c.Csv == "" {
-		csvNames := []string{"CNA", "KubeVirt", "SSP", "CDI", "NMO", "HPP", "VM Import"}
-		log.Panicf("ERROR: the %s CSV was empty", csvNames[i])
+		log.Panicf("ERROR: the %s CSV was empty", csvName)
 	}
 	csvBytes := []byte(c.Csv)
 
 	csvStruct := &csvv1alpha1.ClusterServiceVersion{}
 
-	panicOnError(yaml.Unmarshal(csvBytes, csvStruct))
+	panicOnError(yaml.Unmarshal(csvBytes, csvStruct), "failed to unmarshal the CSV for", csvName)
 
 	strategySpec := csvStruct.Spec.InstallStrategy.StrategySpec
 
@@ -384,12 +386,12 @@ func processOneCsv(c util.CsvWithComponent, i int, installStrategyBase *csvv1alp
 		csvBaseAlmString = "[" + csvBaseAlmString + "]"
 	}
 
-	panicOnError(json.Unmarshal([]byte(csvBaseAlmString), &baseAlmcrs))
-	panicOnError(json.Unmarshal([]byte(csvStructAlmString), &structAlmcrs))
+	panicOnError(json.Unmarshal([]byte(csvBaseAlmString), &baseAlmcrs), "failed to unmarshal the example from base from base csv for", csvName, "csvBaseAlmString:", csvBaseAlmString)
+	panicOnError(json.Unmarshal([]byte(csvStructAlmString), &structAlmcrs), "failed to unmarshal the example from base from struct csv for", csvName, "csvStructAlmString:", csvStructAlmString)
 
 	baseAlmcrs = append(baseAlmcrs, structAlmcrs...)
 	almB, err := json.Marshal(baseAlmcrs)
-	panicOnError(err)
+	panicOnError(err, "failed to marshal the combined example for", csvName)
 	csvBase.Annotations[almExamplesAnnotation] = string(almB)
 
 	if !*ignoreComponentsRelatedImages {
@@ -421,6 +423,18 @@ func applyOverrides(csvBase *csvv1alpha1.ClusterServiceVersion) {
 	}
 }
 
+func setSupported(csvBase *csvv1alpha1.ClusterServiceVersion) {
+	if csvBase.Labels == nil {
+		csvBase.Labels = make(map[string]string)
+	}
+	for _, ele := range supported_archs {
+		csvBase.Labels[operatorFrameworkPrefix+ele] = supported
+	}
+	for _, ele := range supported_os {
+		csvBase.Labels[operatorFrameworkPrefix+ele] = supported
+	}
+}
+
 func getInitialCsvList() []util.CsvWithComponent {
 	return []util.CsvWithComponent{
 		{
@@ -447,10 +461,6 @@ func getInitialCsvList() []util.CsvWithComponent {
 			Csv:       *hppCsv,
 			Component: hcoutil.AppComponentStorage,
 		},
-		{
-			Csv:       *vmImportCsv,
-			Component: hcoutil.AppComponentImport,
-		},
 	}
 }
 
@@ -488,25 +498,22 @@ func getCsvBaseParams(replaces string, version semver.Version) *components.CSVBa
 
 func getDeploymentParams() *components.DeploymentOperatorParams {
 	return &components.DeploymentOperatorParams{
-		Namespace:           *namespace,
-		Image:               *operatorImage,
-		WebhookImage:        *webhookImage,
-		CliDownloadsImage:   *cliDownloadsImage,
-		ImagePullPolicy:     "IfNotPresent",
-		ConversionContainer: *imsConversionImage,
-		VmwareContainer:     *imsVMWareImage,
-		VirtIOWinContainer:  *kvVirtIOWinImage,
-		Smbios:              *smbios,
-		Machinetype:         *machinetype,
-		HcoKvIoVersion:      *hcoKvIoVersion,
-		KubevirtVersion:     *kubevirtVersion,
-		CdiVersion:          *cdiVersion,
-		CnaoVersion:         *cnaoVersion,
-		SspVersion:          *sspVersion,
-		NmoVersion:          *nmoVersion,
-		HppoVersion:         *hppoVersion,
-		VMImportVersion:     *vmImportVersion,
-		Env:                 envVars,
+		Namespace:          *namespace,
+		Image:              *operatorImage,
+		WebhookImage:       *webhookImage,
+		CliDownloadsImage:  *cliDownloadsImage,
+		ImagePullPolicy:    "IfNotPresent",
+		VirtIOWinContainer: *kvVirtIOWinImage,
+		Smbios:             *smbios,
+		Machinetype:        *machinetype,
+		HcoKvIoVersion:     *hcoKvIoVersion,
+		KubevirtVersion:    *kubevirtVersion,
+		CdiVersion:         *cdiVersion,
+		CnaoVersion:        *cnaoVersion,
+		SspVersion:         *sspVersion,
+		NmoVersion:         *nmoVersion,
+		HppoVersion:        *hppoVersion,
+		Env:                envVars,
 	}
 }
 
@@ -547,9 +554,14 @@ func addRelatedImage(images []csvv1alpha1.RelatedImage, image string) []csvv1alp
 	return appendRelatedImageIfMissing(images, ri)
 }
 
-func panicOnError(err error) {
+func panicOnError(err error, info ...string) {
 	if err != nil {
-		log.Println("Error!", err)
+		moreInfo := ""
+		if len(info) > 0 {
+			moreInfo = strings.Join(info, " ")
+		}
+
+		log.Println("Error!", err, moreInfo)
 		panic(err)
 	}
 }
